package {

	import Box2DAS.Collision.Shapes.b2CircleShape;
	import Box2DAS.Common.V2;
	import Box2DAS.Dynamics.ContactEvent;
	import Box2DAS.Dynamics.b2FixtureDef;

	import com.citrusengine.objects.PhysicsObject;

	import org.osflash.signals.Signal;

	/**
	 * @author Aymeric
	 */
	public class Ball extends PhysicsObject {

		public var onBeginContact:Signal;
		public var onEndContact:Signal;

		private var _radius:Number;
		private var _linearVelocity:V2;

		private var _afterFirstBounce:Boolean = false;

		public function Ball(name:String, params:Object = null) {

			super(name, params);

			onBeginContact = new Signal(ContactEvent);
			onEndContact = new Signal(ContactEvent);

			_linearVelocity = _body.GetLinearVelocity();
			_linearVelocity.y = 5;
		}

		override public function destroy():void {

			onBeginContact.removeAll();
			onEndContact.removeAll();
			super.destroy();
		}

		override protected function createShape():void {

			_shape = new b2CircleShape();
			b2CircleShape(_shape).m_radius = _radius;
		}

		override public function update(timeDelta:Number):void {

			if (_afterFirstBounce == true) {
				_linearVelocity = _body.GetLinearVelocity();
			}
			_body.SetLinearVelocity(_linearVelocity);
		}

		override protected function createFixture():void {
			
			super.createFixture();
			_fixture.m_reportBeginContact = true;
			_fixture.m_reportEndContact = true;
			_fixture.addEventListener(ContactEvent.BEGIN_CONTACT, handleBeginContact);
			_fixture.addEventListener(ContactEvent.END_CONTACT, handleEndContact);
		}

		override protected function defineFixture():void {
			
			_fixtureDef = new b2FixtureDef();
			_fixtureDef.shape = _shape;
			_fixtureDef.density = 1;
			_fixtureDef.friction = 0;
			_fixtureDef.restitution = 1;
		}

		private function handleBeginContact(cEvt:ContactEvent):void {

			onBeginContact.dispatch(cEvt);

			var diff:Number;
			var ratio:Number;
			var speed:Number;

			if (cEvt.other.GetBody().GetUserData() is Paddle) {
				_afterFirstBounce = true;
				// get a reference to the paddle
				var paddle:Paddle = cEvt.other.GetBody().GetUserData() as Paddle;

				// only change collisions that are against the top surface of the paddle (let it shank off if it hits an edge).
				var collisionNormal:V2 = cEvt.getWorldManifold().normal;
				if (collisionNormal.x == 0 && collisionNormal.y == 1) {
					diff = x - paddle.x;
					// x distance between ball and paddle.
					ratio = diff / (paddle.width / 2);

					// distance as a ratio (between -1 and 1)
					if (ratio < -1 || ratio > 1) // it will shank.
						return;

					// Create a new velocity vector and set it to the ball's current speed.
					// var velocity:V2 = _body.GetLinearVelocity();
					_linearVelocity = _body.GetLinearVelocity();
					speed = _linearVelocity.length();

					// set the ball to be going straight up at the same speed as it came.
					_linearVelocity = new V2(0, -1);
					_linearVelocity.normalize(speed);

					// rotate the velocity angle between a specific range (I chose 1 radian) as a proportion ball's distance from center of paddle.
					_linearVelocity.rotate(1 * ratio);
					_body.SetLinearVelocity(_linearVelocity);
				}
			}
		}

		private function handleEndContact(cEvt:ContactEvent):void {
			onEndContact.dispatch(cEvt);
		}

		public function get radius():Number {
			return _radius * _box2D.scale;
		}

		public function set radius(value:Number):void {
			_radius = value / _box2D.scale;
		}
	}
}
